#include "box.hh"
#include "common.hh"
#include "params.hh"

Box::Box (const Vector& low_, const Vector& high_): low(low_), high(high_) {}

const void Box::inside (Vector& point, const Vector& old) const {
  if ((point.x() > low.x()) && (point.x() < high.x()) &&
      (point.y() > low.y()) && (point.y() < high.y()) && 
      (point.z() > low.z()) && (point.z() < high.z())) {
    double lambda;
    if ((old.x() >= low.x()) && (old.x() <= high.x())) {
      if ((old.y() >= low.y()) && (old.y() <= high.y())) {
        if (old.z() <= low.z()) {
          lambda = (low.z()-old.z());
          if (lambda >= EPSILON)
            lambda /= (point.z()-old.z());
          else
            lambda = 0;
        } else {
          lambda = (high.z()-old.z());
          if (lambda <= -EPSILON)
            lambda /= (point.z()-old.z());
          else
            lambda = 0;
        }
      } else {
        if ((old.z() >= low.z()) && (old.z() <= high.z())) {
          if (old.y() <= low.y()) {
            lambda = (low.y()-old.y());
            if (lambda >= EPSILON)
              lambda /= (point.y()-old.y());
            else
              lambda = 0;
          } else {
            lambda = (high.y()-old.y());
            if (lambda <= -EPSILON)
              lambda /= (point.y()-old.y());
            else
              lambda = 0;
          }
        } else {
          double lambda2;
          if (old.z() <= low.z()) {
            lambda = (low.z()-old.z());
            if (lambda >= EPSILON)
              lambda /= (point.z()-old.z());
            else
              lambda = 0;
          } else {
            lambda = (high.z()-old.z());
            if (lambda <= -EPSILON)
              lambda /= (point.z()-old.z());
            else
              lambda = 0;
          }
          if (old.y() <= low.y()) {
            lambda2 = (low.y()-old.y());
            if (lambda2 >= EPSILON)
              lambda2 /= (point.y()-old.y());
            else
              lambda2 = 0;
          } else {
            lambda2 = (high.y()-old.y());
            if (lambda2 <= -EPSILON)
              lambda2 /= (point.y()-old.y());
            else
              lambda2 = 0;
          }
          if (lambda2 > lambda) lambda = lambda2;
        }
      }
    } else {
      if ((old.y() >= low.y()) && (old.y() <= high.y())) {
        if ((old.z() >= low.z()) && (old.z() <= high.z())) {
          if (old.x() <= low.x()) {
            lambda = (low.x()-old.x());
            if (lambda >= EPSILON)
              lambda /= (point.x()-old.x());
            else
              lambda = 0;
          } else {
            lambda = (high.x()-old.x());
            if (lambda <= -EPSILON)
              lambda /= (point.x()-old.x());
            else
              lambda = 0;
          }
        } else {
          double lambda2;
          if (old.z() <= low.z()) {
            lambda = (low.z()-old.z());
            if (lambda >= EPSILON)
              lambda /= (point.z()-old.z());
            else
              lambda = 0;
          } else {
            lambda = (high.z()-old.z());
            if (lambda <= -EPSILON)
              lambda /= (point.z()-old.z());
            else
              lambda = 0;
          }
          if (old.x() <= low.x()) {
            lambda2 = (low.x()-old.x());
            if (lambda2 >= EPSILON)
              lambda2 /= (point.x()-old.x());
            else
              lambda2 = 0;
          } else {
            lambda2 = (high.x()-old.x());
            if (lambda2 <= -EPSILON)
              lambda2 /= (point.x()-old.x());
            else
              lambda2 = 0;
          }
          if (lambda2 > lambda) lambda = lambda2;
        }
      } else {
        if ((old.z() >= low.z()) && (old.z() <= high.z())) {
          double lambda2;
          if (old.y() <= low.y()) {
            lambda = (low.y()-old.y());
            if (lambda >= EPSILON)
              lambda /= (point.y()-old.y());
            else
              lambda = 0;
          } else {
            lambda = (high.y()-old.y());
            if (lambda <= -EPSILON)
              lambda /= (point.y()-old.y());
            else
              lambda = 0;
          }
          if (old.x() <= low.x()) {
            lambda2 = (low.x()-old.x());
            if (lambda2 >= EPSILON)
              lambda2 /= (point.x()-old.x());
            else
              lambda2 = 0;
          } else {
            lambda2 = (high.x()-old.x());
            if (lambda2 <= -EPSILON)
              lambda2 /= (point.x()-old.x());
            else
              lambda2 = 0;
          }
          if (lambda2 > lambda) lambda = lambda2;
        } else {
          double lambda2, lambda3;
          if (old.z() <= low.z()) {
            lambda = (low.z()-old.z());
            if (lambda >= EPSILON)
              lambda /= (point.z()-old.z());
            else
              lambda = 0;
          } else {
            lambda = (high.z()-old.z());
            if (lambda <= -EPSILON)
              lambda /= (point.z()-old.z());
            else
              lambda = 0;
          }
          if (old.y() <= low.y()) {
            lambda2 = (low.y()-old.y());
            if (lambda2 >= EPSILON)
              lambda2 /= (point.y()-old.y());
            else
              lambda2 = 0;
          } else {
            lambda2 = (high.y()-old.y());
            if (lambda2 <= -EPSILON)
              lambda2 /= (point.y()-old.y());
            else
              lambda2 = 0;
          }
          if (old.x() <= low.x()) {
            lambda3 = (low.x()-old.x());
            if (lambda3 >= EPSILON)
              lambda3 /= (point.x()-old.x());
            else
              lambda3 = 0;
          } else {
            lambda3 = (high.x()-old.x());
            if (lambda3 <= -EPSILON)
              lambda3 /= (point.x()-old.x());
            else
              lambda3 = 0;
          }
          if (lambda2 > lambda) lambda = lambda2;
          if (lambda3 > lambda) lambda = lambda3;
        }
      }
    }
    point = old+lambda*(point-old);
  }
}

const void Box::adjust (Vector& force, const Vector& point) const {
  if ((point.x() <= low.x()-EPSILON) || (point.x() >= high.x() + EPSILON) ||
      (point.y() <= low.y()-EPSILON) || (point.y() >= high.y() + EPSILON) ||
      (point.z() <= low.z()-EPSILON) || (point.z() >= high.z() + EPSILON))
    return;
  if (point.z() >= high.z()) {
    if (force.z() < 0) force.z (0);
    Vector tmp = force;
    tmp.z(0);
    if (tmp.length() < friction+EPSILON)
      force -= tmp;
    else
      force -= (friction/tmp.length())*tmp;
  } else if (point.z() <= low.z()) {
    if (force.z() > 0) force.z (0);
    Vector tmp = force;
    tmp.z(0);
    if (tmp.length() < friction+EPSILON)
      force -= tmp;
    else
      force -= (friction/tmp.length())*tmp;
  } else {
    if (point.x() >= high.x()) {
      if (force.x() < 0) force.x (0);
      Vector tmp = force;
      tmp.x(0);
      if (tmp.length() < friction+EPSILON)
        force -= tmp;
      else
        force -= (friction/tmp.length())*tmp;
    } else if (point.x() <= low.x()) {
      if (force.x() > 0) force.x (0);
      Vector tmp = force;
      tmp.x(0);
      if (tmp.length() < friction+EPSILON)
        force -= tmp;
      else
        force -= (friction/tmp.length())*tmp;
    } else {
      if (point.y() >= high.y()) {
        if (force.y() < 0) force.y (0);
        Vector tmp = force;
        tmp.y(0);
        if (tmp.length() < friction+EPSILON)
          force -= tmp;
        else
          force -= (friction/tmp.length())*tmp;
      } else {
        if (force.y() > 0) force.y (0);
        Vector tmp = force;
        tmp.y(0);
        if (tmp.length() < friction+EPSILON)
          force -= tmp;
        else
          force -= (friction/tmp.length())*tmp;
      }
    }
  }
}
